@@ -1006,6 +1006,19 @@ class RfbProto {
     boolean down = (evt.getID() == KeyEvent.KEY_PRESSED);
 
     int key;
+	if (viewer.debugKeyboard) {
+		System.out.println("----------------------------------------");
+		System.out.println("evt.getKeyChar:      " + evt.getKeyChar());
+		System.out.println("getKeyText:          " + KeyEvent.getKeyText(evt.getKeyCode()));
+		System.out.println("evt.getKeyCode:      " + evt.getKeyCode());
+		System.out.println("evt.getID:           " + evt.getID());
+		System.out.println("evt.getKeyLocation:  " + evt.getKeyLocation());
+		System.out.println("evt.isActionKey:     " + evt.isActionKey());
+		System.out.println("evt.isControlDown:   " + evt.isControlDown());
+		System.out.println("evt.getModifiers:    " + evt.getModifiers());
+		System.out.println("getKeyModifiersText: " + KeyEvent.getKeyModifiersText(evt.getModifiers()));
+		System.out.println("evt.paramString:     " + evt.paramString());
+	}
     if (evt.isActionKey()) {
 
       //
@@ -1039,6 +1052,13 @@ class RfbProto {
         return;
       }
 
+      if(key == 0xffc2 && viewer.mapF5_to_atsign) {
+	 if (viewer.debugKeyboard) {
+	    System.out.println("Mapping: F5 -> AT ");
+	 }
+      	 key = 0x40;
+      }
+
     } else {
 
       //
@@ -1050,6 +1070,7 @@ class RfbProto {
 
       key = keyChar;
 
+
       if (key < 0x20) {
         if (evt.isControlDown()) {
           key += 0x60;
@@ -1135,6 +1156,16 @@ class RfbProto {
   int oldModifiers = 0;
 
   void writeModifierKeyEvents(int newModifiers) {
+    if(viewer.forbid_Ctrl_Alt) {
+	if ((newModifiers & CTRL_MASK) != 0 && (newModifiers & ALT_MASK) != 0) {
+		int orig = newModifiers;
+		newModifiers &= ~ALT_MASK;
+		newModifiers &= ~CTRL_MASK;
+		if (viewer.debugKeyboard) {
+			System.out.println("Ctrl+Alt modifiers: " + orig + " -> " + newModifiers);
+		}
+	}
+    }
     if ((newModifiers & CTRL_MASK) != (oldModifiers & CTRL_MASK))
       writeKeyEvent(0xffe3, (newModifiers & CTRL_MASK) != 0);
 
