@@ -1,5 +1,5 @@
 /*
-   Copyright (C) 2002-2009 Karl J. Runge <runge@karlrunge.com> 
+   Copyright (C) 2002-2010 Karl J. Runge <runge@karlrunge.com> 
    All rights reserved.
 
 This file is part of x11vnc.
@@ -1114,7 +1114,8 @@ void switch_to_xkb_if_better(void) {
 	n = k;
 
 	XFree_wr(keymap);
-	if (missing_noxkb == 0 && syms_gt_4 >= 8) {
+	if (missing_noxkb == 0 && syms_per_keycode > 4 && syms_gt_4 >= 0) {
+		/* we used to have syms_gt_4 >= 8, now always on. */
 		if (! raw_fb_str) {
 			rfbLog("\n");
 			rfbLog("XKEYBOARD: number of keysyms per keycode %d is greater\n", syms_per_keycode);
@@ -1123,6 +1124,7 @@ void switch_to_xkb_if_better(void) {
 			rfbLog("  If this makes the key mapping worse you can\n");
 			rfbLog("  disable it with the \"-noxkb\" option.\n");
 			rfbLog("  Also, remember \"-remap DEAD\" for accenting characters.\n");
+			rfbLog("\n");
 		}
 
 		use_xkb_modtweak = 1;
@@ -1135,6 +1137,7 @@ void switch_to_xkb_if_better(void) {
 			rfbLog("  Not automatically switching to -xkb mode.\n");
 			rfbLog("  If some keys still cannot be typed, try using -xkb.\n");
 			rfbLog("  Also, remember \"-remap DEAD\" for accenting characters.\n");
+			rfbLog("\n");
 		}
 		return;
 	}
@@ -1217,6 +1220,7 @@ void switch_to_xkb_if_better(void) {
 		rfbLog("  Also, remember \"-remap DEAD\" for accenting"
 		    " characters.\n");
 	}
+	rfbLog("\n");
 }
 
 /* sets up all the keymapping info via Xkb API */
@@ -1321,7 +1325,7 @@ xkbmodifiers[]    For the KeySym bound to this (keycode,group,level) store
 	 * loop over all possible (keycode, group, level) triples
 	 * and record what we find for it:
 	 */
-	if (debug_keyboard > 1) {
+	if (debug_keyboard) {
 		rfbLog("initialize_xkb_modtweak: XKB keycode -> keysyms "
 		    "mapping info:\n");
 	}
@@ -1498,7 +1502,7 @@ xkbmodifiers[]    For the KeySym bound to this (keycode,group,level) store
 				xkbignore[kc][grp][lvl] = mods_save | ms2;
 			}
 
-			if (debug_keyboard > 1) {
+			if (debug_keyboard) {
 				char *str;
 				fprintf(stderr, "  %03d  G%d L%d  mod=%s ",
 				    kc, grp+1, lvl+1, bitprint(ms, 8));
@@ -1556,7 +1560,7 @@ xkbmodifiers[]    For the KeySym bound to this (keycode,group,level) store
 		}
 		free(str);
 	}
-	if (debug_keyboard > 1) {
+	if (debug_keyboard) {
 		fprintf(stderr, "grp_max=%d lvl_max=%d\n", grp_max, lvl_max);
 	}
 }
@@ -2894,9 +2898,9 @@ static void pipe_keyboard(rfbBool down, rfbKeySym keysym, rfbClientPtr client) {
 			t[1] = '\0';
 			if (sscanf(t, "%d", &butt) == 1) {
 				mask = 1<<(butt-1);
-				pointer(mask, x, y, client);
+				pointer_event(mask, x, y, client);
 				mask = 0;
-				pointer(mask, x, y, client);
+				pointer_event(mask, x, y, client);
 			}
 			b++;
 		}
